"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var table_dispatcher_exports = {};
__export(table_dispatcher_exports, {
  makeTableDispatcher: () => makeTableDispatcher
});
module.exports = __toCommonJS(table_dispatcher_exports);
var import_fs = require("fs");
var import_node_path = __toESM(require("node:path"));
var import_js_yaml = __toESM(require("js-yaml"));
var import_combine_fragments = require("./combine-fragments");
var import_lambda = require("./lambda");
var import_log_group = require("./log-group");
function makeTableDispatcher(config, {
  batchSize,
  buildProperties,
  codeUri,
  dependenciesModuleId,
  functionName,
  libImportPath,
  maximumRetryAttempts,
  memorySize,
  nested,
  nestedStackLocation,
  nestedStackTemplatePath,
  outputPath,
  tableName,
  timeout
}) {
  (0, import_lambda.writeLambda)(
    outputPath,
    `// This file is generated. Do not edit by hand.

import {expandTableNames,makeDynamoDBStreamDispatcher} from '${libImportPath}';
import * as dependencies from '${dependenciesModuleId}';

expandTableNames();

export const handler = makeDynamoDBStreamDispatcher({
  ...dependencies,
  tableName: '${tableName}',
});
`
  );
  if (nested) {
    writeTemplate(config, nestedStackTemplatePath);
    const stack = {
      Properties: {
        Location: nestedStackLocation,
        Parameters: {
          BatchSize: batchSize,
          CodeUri: codeUri,
          EventBus: "default",
          MaximumRetryAttempts: maximumRetryAttempts,
          MemorySize: memorySize,
          StreamArn: { "Fn::GetAtt": [tableName, "StreamArn"] },
          TableNames: { Type: "String" },
          Timeout: timeout
        }
      },
      Type: "AWS::Serverless::Application"
    };
    return {
      Resources: {
        [`Nested${functionName}Stack`]: stack
      }
    };
  }
  return (0, import_combine_fragments.combineFragments)((0, import_log_group.makeLogGroup)({ functionName }), {
    Resources: {
      [functionName]: {
        Metadata: {
          BuildMethod: "esbuild",
          BuildProperties: buildProperties
        },
        Properties: {
          CodeUri: codeUri,
          Events: {
            Stream: {
              Properties: {
                BatchSize: batchSize,
                FunctionResponseTypes: ["ReportBatchItemFailures"],
                MaximumRetryAttempts: maximumRetryAttempts,
                StartingPosition: "TRIM_HORIZON",
                Stream: { "Fn::GetAtt": [tableName, "StreamArn"] }
              },
              Type: "DynamoDB"
            }
          },
          MemorySize: memorySize,
          Policies: [
            "AWSLambdaBasicExecutionRole",
            "AWSLambda_ReadOnlyAccess",
            "AWSXrayWriteOnlyAccess",
            "CloudWatchLambdaInsightsExecutionRolePolicy",
            { CloudWatchPutMetricPolicy: {} },
            {
              EventBridgePutEventsPolicy: {
                EventBusName: "default"
              }
            }
          ],
          Timeout: timeout
        },
        Type: "AWS::Serverless::Function"
      }
    }
  });
}
function writeTemplate(config, templatePath) {
  const functionName = "Function";
  const fn = {
    Metadata: {
      BuildMethod: "esbuild",
      buildProperties: {
        EntryPoints: ["./index"],
        External: config.buildProperties.external,
        Minify: config.buildProperties.minify,
        Sourcemap: config.buildProperties.sourcemap,
        Target: config.buildProperties.target
      }
    },
    Properties: {
      // @ts-expect-error - typedef does not know about intrinsic functions
      CodeUri: { Ref: "CodeUri" },
      Environment: {
        Variables: {
          // TableNames will get set after-the-fact the the main plugin script
          // before writing the template.
          FOUNDATION_TABLE_NAMES: { Ref: "TableNames" }
        }
      },
      Events: {
        Stream: {
          Properties: {
            BatchSize: { Ref: "BatchSize" },
            FunctionResponseTypes: ["ReportBatchItemFailures"],
            MaximumRetryAttempts: { Ref: "MaximumRetryAttempts" },
            StartingPosition: "TRIM_HORIZON",
            Stream: { Ref: "StreamArn" }
          },
          Type: "DynamoDB"
        }
      },
      MemorySize: { Ref: "MemorySize" },
      Policies: [
        "AWSLambdaBasicExecutionRole",
        "AWSLambda_ReadOnlyAccess",
        "AWSXrayWriteOnlyAccess",
        "CloudWatchLambdaInsightsExecutionRolePolicy",
        { CloudWatchPutMetricPolicy: {} },
        {
          EventBridgePutEventsPolicy: {
            EventBusName: { Ref: "EventBus" }
          }
        }
      ],
      Timeout: { Ref: "Timeout" }
    },
    Type: "AWS::Serverless::Function"
  };
  const fragments = (0, import_combine_fragments.combineFragments)((0, import_log_group.makeLogGroup)({ functionName }), {
    Globals: {
      Function: {
        Handler: "index.handler",
        Runtime: "nodejs18.x",
        Tracing: "Active"
      }
    },
    Outputs: {
      FunctionName: {
        Export: {
          // eslint-disable-next-line no-template-curly-in-string
          Name: { "Fn::Sub": "${AWS::StackName}-FunctionName" }
        },
        Value: { Ref: "Function" }
      }
    },
    Parameters: {
      BatchSize: {
        Type: "Number"
      },
      CodeUri: {
        Type: "String"
      },
      EventBus: {
        Type: "String"
      },
      MaximumRetryAttempts: {
        Type: "Number"
      },
      MemorySize: {
        Type: "Number"
      },
      StreamArn: {
        Type: "String"
      },
      TableNames: {
        Type: "String"
      },
      Timeout: {
        Type: "Number"
      }
    },
    Resources: {
      [functionName]: fn
    }
  });
  const tpl = {
    ...fragments,
    AWSTemplateFormatVersion: "2010-09-09",
    Transform: "AWS::Serverless-2016-10-31"
  };
  const { format } = config.outputConfig;
  const out = format === "json" ? JSON.stringify(tpl, null, 2) : import_js_yaml.default.dump(tpl, config.outputConfig.yamlConfig);
  const dirname = import_node_path.default.dirname(templatePath);
  (0, import_fs.mkdirSync)(dirname, { recursive: true });
  (0, import_fs.writeFileSync)(templatePath, out);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  makeTableDispatcher
});
