"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var plugin_exports = {};
__export(plugin_exports, {
  addToSchema: () => import_foundation_intermediate_representation.schema,
  plugin: () => plugin
});
module.exports = __toCommonJS(plugin_exports);
var import_assert = __toESM(require("assert"));
var import_fs = __toESM(require("fs"));
var import_js_yaml = __toESM(require("js-yaml"));
var import_js_yaml_cloudformation_schema = require("js-yaml-cloudformation-schema");
var import_foundation_parser = require("@code-like-a-carpenter/foundation-parser");
var import_graphql_codegen_helpers = require("@code-like-a-carpenter/graphql-codegen-helpers");
var import_cdc = require("./cdc");
var import_config = require("./config");
var import_combine_fragments = require("./fragments/combine-fragments");
var import_table = require("./table");
var import_foundation_intermediate_representation = require("@code-like-a-carpenter/foundation-intermediate-representation");
function getInitialTemplate({
  sourceTemplate
}) {
  if (sourceTemplate) {
    const raw = import_fs.default.readFileSync(sourceTemplate, "utf8");
    try {
      return JSON.parse(raw);
    } catch {
      return import_js_yaml.default.load(raw, {
        schema: import_js_yaml_cloudformation_schema.CLOUDFORMATION_SCHEMA
      });
    }
  }
  return {
    AWSTemplateFormatVersion: "2010-09-09",
    Resources: {},
    Transform: ["AWS::Serverless-2016-10-31", "AWS::LanguageExtensions"]
  };
}
const plugin = (0, import_graphql_codegen_helpers.makePlugin)(
  import_config.ConfigSchema,
  (schema2, documents, config, info) => {
    const outputFile = info?.outputFile;
    (0, import_assert.default)(outputFile, "outputFile is required");
    const { models, tables } = (0, import_foundation_parser.parse)(schema2, documents, config, info);
    const allResources = (0, import_combine_fragments.combineFragments)(
      ...tables.map(
        (table) => (0, import_combine_fragments.combineFragments)((0, import_cdc.defineTableCdc)(table, config), (0, import_table.defineTable)(table))
      ),
      ...models.flatMap(
        (model) => model.changeDataCaptureConfig.map((cdc) => {
          const { type } = cdc;
          switch (type) {
            case "ENRICHER":
              return (0, import_cdc.defineEnricher)(config, model, cdc);
            case "REACTOR":
              return (0, import_cdc.defineReactor)(config, model, cdc);
            default:
              throw new Error(`Unexpected CDC type ${type}`);
          }
        })
      )
    );
    const initialTemplate = getInitialTemplate(config);
    const tpl = {
      ...initialTemplate,
      Conditions: {
        ...initialTemplate.Conditions,
        ...allResources.Conditions
      },
      Globals: {
        Function: {
          Handler: "index.handler",
          MemorySize: 256,
          Runtime: "nodejs18.x",
          Timeout: 30,
          Tracing: "Active",
          ...initialTemplate?.Globals?.Function,
          Environment: {
            // @ts-expect-error - typedef treats `Environment` as `unknown`
            ...initialTemplate?.Globals?.Function?.Environment,
            Variables: {
              // @ts-expect-error - typedef treats `Environment` as `unknown`
              ...initialTemplate?.Globals?.Function?.Environment?.Variables,
              // @ts-expect-error - typedef treats `Environment` as `unknown`
              ...allResources?.Globals?.Function?.Environment?.Variables
            }
          }
        }
      },
      Outputs: {
        ...initialTemplate.Outputs,
        ...allResources.Outputs
      },
      Parameters: {
        ...initialTemplate.Parameters,
        ...allResources.Parameters,
        StageName: {
          AllowedValues: ["development", "production", "test"],
          Default: "development",
          Description: "The name of the stage",
          Type: "String"
        }
      },
      Resources: {
        ...initialTemplate.Resources,
        ...allResources.Resources
      }
    };
    const variables = tpl?.Globals?.Function?.Environment?.Variables ?? {};
    const tablesNames = Object.keys(variables).filter((name) => name.startsWith("TABLE_")).reduce((acc, name) => {
      acc[name] = variables[name];
      delete variables[name];
      return acc;
    }, {});
    if (Object.keys(tablesNames).length) {
      variables.FOUNDATION_TABLE_NAMES = { "Fn::ToJsonString": tablesNames };
    }
    Object.entries(tpl.Resources).filter(
      ([, resource]) => resource.Type === "AWS::Serverless::Application"
    ).forEach(([, resource]) => {
      (0, import_assert.default)(resource.Type === "AWS::Serverless::Application");
      resource.Properties = {
        ...resource.Properties,
        Parameters: {
          ...resource.Properties?.Parameters ?? {},
          TableNames: { "Fn::ToJsonString": tablesNames }
        }
      };
    });
    const { format } = config.outputConfig;
    if (format === "json") {
      return JSON.stringify(tpl, null, 2);
    }
    return import_js_yaml.default.dump(tpl, config.outputConfig.yamlConfig);
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  addToSchema,
  plugin
});
